#
# This is a level file. The lines beginning with a # are
# comments (ignored). See HOWTO.Modify for a better description.
#

# 1
{
	maze = m21.map
	tile = tile.map
	fruit =  cherry.map
	depth = 22
	regen_wait = 200
	fruit_val = 100
	fruit_quota = 10
	bg = BLACK
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 2
{
	maze = m1.map
	tile = tile2.map
	fruit = berry.map
	depth = 18
	regen_wait = 200
	fruit_val = 200
	fruit_quota = 10
	bg = BLACK
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}


# 3
{
	maze = m2.map
	tile = tile3.map
	fruit = can.map
	depth = 22
	regen_wait = 200
	fruit_val = 300
	fruit_quota = 10
	bg = BLACK
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 4
{
	maze = m2.5.map
	tile = tile4.map
	fruit = apple.map
	depth = 18
	regen_wait = 200
	fruit_val = 500
	fruit_quota = 10
	bg = BLACK
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 5
{
	maze = m8.map
	tile = tile5.map
	fruit = apple.map
	depth = 21
	regen_wait = 200
	fruit_val = 500
	fruit_quota = 10
	bg = BLACK
	dot = GREEN
	bigdot = WHITE
	ghost_blue = 100
	ghost_flash = 60
}

# 6
{
	maze = m6.map
	tile = tile10.map
	fruit = shield.map
	depth = 18
	regen_wait = 200
	fruit_val = 1000
	fruit_quota = 10
	bg = BLACK
	dot = WHITE
	bigdot = LIGHTBLUE
	ghost_blue = 100
	ghost_flash = 60
}

# 7
{
	maze = m7.map
	tile = tile7.map
	fruit = shield.map
	depth = 20
	regen_wait = 200
	fruit_val = 1000
	fruit_quota = 10
	bg = BLUE
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 8
{
	maze = m9.map
	tile = tile13.map
	fruit = tbird.map
	depth = 20
	regen_wait = 200
	fruit_val = 2000
	fruit_quota = 10
	bg = BLACK
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 9
{
	maze = m10.map
	tile = tile9.map
	fruit = tbird.map
	depth = 18
	regen_wait = 200
	fruit_val = 2000
	fruit_quota = 10
	bg = BLACK
	dot = RED
	bigdot = WHITE
	ghost_blue = 100
	ghost_flash = 60
}

# 10
{
	maze = m22.map
	tile = tile8.map
	fruit = chest.map
	depth = 16
	regen_wait = 200
	fruit_val = 5000
	fruit_quota = 10
	bg = BLACK
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}


	
	
	
	

