# 1
{
	maze = m1.map
	tile = tile.map
	fruit = cherry.map
	depth = 25
	fruit_val = 100
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 2
{
	maze = m2.map
	tile = tile.map
	fruit = berry.map
	depth = 25
	fruit_val = 200
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 3
{
	maze = m2.1.map
	tile = tile.map
	fruit = can.map
	depth = 25
	fruit_val = 300
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 4
{
	maze = m2.5.map
	tile = tile.map
	fruit = apple.map
	depth = 25
	fruit_val = 500
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 5
{
	maze = m8.map
	tile = tile.map
	fruit = apple.map
	depth = 25
	fruit_val = 500
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 6
{
	maze = m6.map
	tile = tile.map
	fruit = shield.map
	depth = 25
	fruit_val = 1000
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 7
{
	maze = m3.map
	tile = tile.map
	fruit = shield.map
	depth = 25
	fruit_val = 1000
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 8
{
	maze = m7.map
	tile = tile.map
	fruit = tbird.map
	depth = 25
	fruit_val = 2000
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 9
{
	maze = m5.map
	tile = tile.map
	fruit = tbird.map
	depth = 25
	fruit_val = 2000
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 10
{
	maze = m9.map
	tile = tile.map
	fruit = chest.map
	depth = 25
	fruit_val = 5000
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

# 11
{
	maze = m10.map
	tile = tile.map
	fruit = chest.map
	depth = 25
	fruit_val = 5000
	fruit_quota = 10
	dot = YELLOW
	bigdot = YELLOW
	ghost_blue = 100
	ghost_flash = 60
}

	
	
	
	