/*
   gtools.h

   This file is part of libgtools.
   
   Copyright (C) 1994,1995 Frank McIngvale (frankm@nuance.com)
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _gtools_h_
#define _gtools_h_

#include <gtools/button.h>
#include <gtools/bitmap.h>
#include <gtools/font.h>
#include <gtools/geom.h>
#include <gtools/gr.h>
#include <gtools/grid.h>
#include <gtools/handle.h>
#include <gtools/mouse.h>
#include <gtools/screen.h>
#include <gtools/fmalloc.h>
#include <gtools/input.h>
#include <gtools/util.h>

#endif
